/*
TECNISA - TECNOLOGIA EM INFORMTICA

NOME: APURAC_RELGESMERENDA

DESCRIO:
   PROCEDIMENTO PARA APURAO DE FATURAMENTO DE CLIENTES EM FILIAIS
   QUE UTILIZAM O TIPO DE CONTRATO GESTO MERENDA.

@C:\WEB_505\SQL\End_Conv\ORACLE\6214_FAISTPO.SQL
*/

CREATE OR REPLACE PROCEDURE APURAC_RELGESMERENDA (
      P_CDFILCENTPME IN FILIAL.CDFILIAL%TYPE,
      P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
      P_CDCLIENTE    IN CLIENTE.CDCLIENTE%TYPE,
      P_DATAINI      IN DATE,
      P_DATAFIM      IN DATE,
      P_NRAPURACFAT  IN NOTAFISCAL.NRAPURACFAT%TYPE,
      P_IDGERANOTA   IN VARCHAR DEFAULT 'S',
      P_DTSAIDAPUR   IN DATE DEFAULT NULL) AS

   /* DECLARAO DO TIPO (RECORD) QUE CONTERA OS DADOS DO CURSOR */
   TYPE GMERENDARECTYPE IS RECORD (
      CDFILCENTPME   FILIAL.CDFILCENTPME%TYPE,
      CDFILIAL       FILIAL.CDFILIAL%TYPE,
      CDCLIENTE      CLIENTE.CDCLIENTE%TYPE,
      CDTIPOCARD     TIPOCARD.CDTIPOCARD%TYPE,
      CDSERVFILI     SERVICO.CDSERVFILI%TYPE,
      DTFATURAM      DATE,
      CDPRODUTO      PRODUTO.CDPRODUTO%TYPE,
      VRSERVICO      NUMBER,
      QTEFETIVOS     NUMBER,
      NMSERVICO      SERVICO.NMSERVICO%TYPE);

   /* DECLARAO DO TIPO DO CURSOR */
   TYPE GMERENDACURTYPE IS REF CURSOR RETURN GMERENDARECTYPE;

   /* VARIVEIS QUE CONTERO O CURSOR E O REGISTRO */
   REC_GMERENDA    GMERENDARECTYPE;
   CUR_GMERENDA    GMERENDACURTYPE;

   /* VARIVEIS PARA CONTROLE DE PARAMETROS */
   STIDUTEFEPLA   VARCHAR2(01);

   /* VARIVEIS PARA CONTROLE DE APURAO */
   V_QTEFETSERV   NUMBER;
   V_VRSERVICO    NUMBER;
   V_EXISTE       NUMBER;
   V_NMSERVICO    SERVICO.NMSERVICO%TYPE;

   V_CDSERIE      SERIENF.CDSERIE%TYPE;
   V_TTITENSNF    NUMBER;
   V_CONTITEM     NUMBER;
   V_NRLANCTONF   VARCHAR2(20);
   V_NRSEQITPROD  NUMBER;
   V_CDEMPRESA    FILIAL.CDEMPRESA%TYPE;
   V_CDINSCMUNI   FILIAL.CDINSCMUNI%TYPE;
   V_CDINSCESTA   FILIAL.CDINSCESTA%TYPE;
   V_IDTPIJURCLIE CLIENTE.IDTPIJURCLIE%TYPE;
   V_NRINSJURCLIE CLIENTE.NRINSJURCLIE%TYPE;
   V_IDNOVANOTA   VARCHAR2(01);
   V_PRXLANCESTQ  VARCHAR2(10);
   V_IDUTPROPEMIS VARCHAR2(01);

   V_CDARVPROD    PRODUTO.CDARVPROD%TYPE;
   V_NMPRODUTO    PRODUTO.NMPRODUTO%TYPE;

   /* PROCEDIMENTO QUE ABRE O CURSOS DE ACORDO COM ALGUNS PARAMETROS
    PARAMETROS:
    1. CURSOR A SER ABERTO
    2. INDICADOR DA UTILIZACAO DE EFETIVOS DO PLANEJAMENTO*/

   PROCEDURE OPENGMERENDACURSOR(C_GMERENDA  IN OUT GMERENDACURTYPE,
                                V_UTPLANEJ IN VARCHAR2) IS
   BEGIN
      OPEN C_GMERENDA FOR
         SELECT CDFILCENTPME, CDFILIAL, CDCLIENTE, CDTIPOCARD, CDSERVFILI, DTFATURAM, 
                CDPRODUTO,
                VRSERVICO,
                QTEFETIVOS,
                NMSERVICO                
         FROM
         (
         SELECT CDFILCENTPME, CDFILIAL, CDCLIENTE, CDTIPOCARD, CDSERVFILI, DTFATURAM, 
                CDPRODUTO,
                VALOR_SERVICO_MERENDA(CDFILCENTPME, CDTIPOCARD, CDFILIAL, CDSERVFILI, CDCLIENTE, DTFATURAM) AS VRSERVICO,
                QTEFETIVOS,
                NMSERVICO                
         FROM
         (
         SELECT EFECLIFATMER.CDFILCENTPME, EFECLIFATMER.CDFILIAL,
                EFECLIFATMER.CDCLIENTE, EFECLIFATMER.CDTIPOCARD, EFECLIFATMER.CDSERVFILI, EFECLIFATMER.DTEFETFATMER AS DTFATURAM,
                SERVICO.CDPRODUTO,
                SUM(EFECLIFATMER.QTCOMEFATMER) AS QTEFETIVOS, 
                SERVICO.NMSERVICO
           FROM EFECLIFATMER, SERVCLIEPME, SERVICO
          WHERE EFECLIFATMER.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME
            AND EFECLIFATMER.CDFILIAL = SERVCLIEPME.CDFILIAL
            AND EFECLIFATMER.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD
            AND EFECLIFATMER.CDCLIENTE = SERVCLIEPME.CDCLIENTE
            AND EFECLIFATMER.CDSERVFILI = SERVCLIEPME.CDSERVFILI
            AND SERVICO.CDFILIAL = EFECLIFATMER.CDFILIAL
            AND SERVICO.CDSERVFILI = EFECLIFATMER.CDSERVFILI
            AND EFECLIFATMER.DTEFETFATMER BETWEEN P_DATAINI AND P_DATAFIM
            AND V_UTPLANEJ = 'N'
            AND EFECLIFATMER.CDFILIAL   = P_CDFILIAL
            AND EFECLIFATMER.CDCLIENTE  = P_CDCLIENTE
         GROUP BY EFECLIFATMER.CDFILCENTPME, EFECLIFATMER.CDFILIAL, EFECLIFATMER.CDCLIENTE,
                  EFECLIFATMER.CDTIPOCARD, EFECLIFATMER.DTEFETFATMER, EFECLIFATMER.CDSERVFILI,
                  SERVICO.CDPRODUTO, SERVICO.NMSERVICO
         HAVING (SUM(EFECLIFATMER.QTCOMEFATMER) <> 0)
         UNION
         SELECT EFETCLIEPME.CDFILCENTPME, EFETCLIEPME.CDFILIAL,
                EFETCLIEPME.CDCLIENTE, EFETCLIEPME.CDTIPOCARD, EFETCLIEPME.CDSERVFILI, EFETCLIEPME.DTEFETCLPME AS DTFATURAM,
                SERVICO.CDPRODUTO,  
                SUM(EFETCLIEPME.QTCOMEFCLPME) AS QTEFETIVOS, 
                SERVICO.NMSERVICO
           FROM EFETCLIEPME, SERVCLIEPME, SERVICO
          WHERE EFETCLIEPME.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME
            AND EFETCLIEPME.CDFILIAL = SERVCLIEPME.CDFILIAL
            AND EFETCLIEPME.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD
            AND EFETCLIEPME.CDCLIENTE = SERVCLIEPME.CDCLIENTE
            AND EFETCLIEPME.CDSERVFILI = SERVCLIEPME.CDSERVFILI
            AND SERVICO.CDFILIAL = EFETCLIEPME.CDFILIAL
            AND SERVICO.CDSERVFILI = EFETCLIEPME.CDSERVFILI
            AND EFETCLIEPME.DTEFETCLPME BETWEEN P_DATAINI AND P_DATAFIM
            AND V_UTPLANEJ = 'S'
            AND EFETCLIEPME.CDFILIAL   = P_CDFILIAL
            AND EFETCLIEPME.CDCLIENTE  = P_CDCLIENTE            
         GROUP BY EFETCLIEPME.CDFILCENTPME, EFETCLIEPME.CDFILIAL, EFETCLIEPME.CDCLIENTE,
                  EFETCLIEPME.CDTIPOCARD, EFETCLIEPME.DTEFETCLPME, EFETCLIEPME.CDSERVFILI,
                  SERVICO.CDPRODUTO, SERVICO.NMSERVICO
         HAVING (SUM(EFETCLIEPME.QTCOMEFCLPME) <> 0)
         ) S )S
         WHERE VRSERVICO <> 0;
   END OPENGMERENDACURSOR;

BEGIN

   SELECT MAX(SUBSTR(DSPARAMFATFI,15,1)) /*UTILIZA EFETIVOS DO PLANEJAMENTO MERENDA*/
   INTO STIDUTEFEPLA
   FROM PARAMFILIAL
   WHERE CDFILIAL = P_CDFILCENTPME;

   SELECT NVL(MAX(IDUTPROPEMIS),'N')
   INTO V_IDUTPROPEMIS
   FROM PARAMGERAL;

   /* APAGA OS REGISTROS DA TRAB_APURAC_GM DE APURACOES ANTERIORES */
   DELETE FROM TRAB_APURAC_GM
   WHERE CDFILCENTAPGM  = P_CDFILCENTPME
     AND CDFILIALAPGM = P_CDFILIAL
     AND NRAPURACGM = P_NRAPURACFAT
     AND CDCLIENTEAPGM = P_CDCLIENTE;


   IF (STIDUTEFEPLA IS NULL) OR (STIDUTEFEPLA = ' ') THEN
      STIDUTEFEPLA := 'N';
      -- NAO UTILIZA EFETIVOS DO PLANEJAMENTO
   END IF;


   /* ABRE O CURSOR */
   OPENGMERENDACURSOR(CUR_GMERENDA, STIDUTEFEPLA);
   LOOP
      FETCH CUR_GMERENDA INTO REC_GMERENDA;
      EXIT WHEN CUR_GMERENDA%NOTFOUND;

      V_VRSERVICO  := REC_GMERENDA.VRSERVICO;
      V_QTEFETSERV := REC_GMERENDA.QTEFETIVOS;
      V_NMSERVICO  := REC_GMERENDA.NMSERVICO;

      /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
         E VALORES CORRIGIDOS */

      SELECT COUNT(*)
        INTO V_EXISTE
        FROM TRAB_APURAC_GM
       WHERE CDFILCENTAPGM   = P_CDFILCENTPME
         AND CDFILIALAPGM   = REC_GMERENDA.CDFILIAL
         AND NRAPURACGM     = P_NRAPURACFAT
         AND CDCLIENTEAPGM  = REC_GMERENDA.CDCLIENTE
         AND CDTIPOCARDAPGM   = REC_GMERENDA.CDTIPOCARD
         AND CDSERVFILIAPGM = REC_GMERENDA.CDSERVFILI
         AND ROUND(VRPRECUNITAPGM,2) = ROUND(REC_GMERENDA.VRSERVICO,2);

      IF (V_EXISTE = 0) THEN

         INSERT INTO TRAB_APURAC_GM
           (CDFILCENTAPGM, CDFILIALAPGM, NRAPURACGM, CDCLIENTEAPGM,
            CDTIPOCARDAPGM, CDSERVFILIAPGM, DTAPURACFATGM, CDPRODUTOAPGM,
            NREFETIVOSAPGM, VRPRECUNITAPGM, NMSERVICOGM)
         VALUES (P_CDFILCENTPME, REC_GMERENDA.CDFILIAL, P_NRAPURACFAT, REC_GMERENDA.CDCLIENTE,
                 REC_GMERENDA.CDTIPOCARD, REC_GMERENDA.CDSERVFILI, P_DATAINI, REC_GMERENDA.CDPRODUTO,
                 REC_GMERENDA.QTEFETIVOS, REC_GMERENDA.VRSERVICO, REC_GMERENDA.NMSERVICO);

      ELSE
         UPDATE TRAB_APURAC_GM
            SET NREFETIVOSAPGM = NREFETIVOSAPGM + V_QTEFETSERV
          WHERE CDFILCENTAPGM    = P_CDFILCENTPME
            AND CDFILIALAPGM   = REC_GMERENDA.CDFILIAL
            AND NRAPURACGM     = P_NRAPURACFAT
            AND CDCLIENTEAPGM  = REC_GMERENDA.CDCLIENTE
            AND CDTIPOCARDAPGM   = REC_GMERENDA.CDTIPOCARD
            AND CDSERVFILIAPGM = REC_GMERENDA.CDSERVFILI
            AND ROUND(VRPRECUNITAPGM,2) = ROUND(REC_GMERENDA.VRSERVICO,2);
      END IF;

      IF STIDUTEFEPLA = 'S' THEN
         UPDATE EFETCLIEPME
            SET CDFILIFATU  = NULL,
                NRAPURACFAT = NULL
          WHERE CDFILCENTPME = REC_GMERENDA.CDFILCENTPME
            AND CDFILIAL    = REC_GMERENDA.CDFILIAL
            AND CDCLIENTE   = P_CDCLIENTE
            AND DTEFETCLPME >= P_DATAINI;
      ELSE

         UPDATE EFECLIFATMER
            SET CDFILIFATU  = NULL,
                NRAPURACFAT = NULL
          WHERE CDFILCENTPME = REC_GMERENDA.CDFILCENTPME AND
                CDFILIAL = REC_GMERENDA.CDFILIAL AND
                CDCLIENTE  = P_CDCLIENTE  AND
                DTEFETFATMER >= P_DATAINI;
      END IF;
   END LOOP;
   CLOSE CUR_GMERENDA;


END;
/

-- EXECUTE APURAC_GESMERENDA('0751', '0712', 'F01162619864', '20/10/2008', '20/10/2008', '0000000001', 'S', '20/10/2008');
-- RAISE_APPLICATION_ERROR(-20501, 'MENSAGEM', TRUE);


-- VERSAO --
BEGIN ATUVEROBJ('APURAC_GESDIRETA','5.05.035'); END;
/

